﻿namespace NHibernate.Test.NHSpecificTest.NH2527 {
   using System;
   using System.Data;
   using NHibernate.Driver;

   public class CustomSQLiteDriver : SQLite20Driver {
      public override IDbCommand CreateCommand() {
         IDbCommand origCmd = base.CreateCommand();
         return new ForwardingDbCommand(origCmd);
      }
   }

   public class ForwardingDbCommand : IDbCommand {
      private readonly IDbCommand _cmd;
      private bool _disposed = false;

      public ForwardingDbCommand(IDbCommand origCmd) {
         _cmd = origCmd;
      }

      private void CheckDisposed() {
         if (_disposed) {
            throw new ObjectDisposedException("ForwardingDbCommand");
         }
      }

      public void Cancel() {
         CheckDisposed();
         _cmd.Cancel();
      }

      public string CommandText {
         get {
            CheckDisposed();
            return _cmd.CommandText;
         }
         set {
            CheckDisposed();
            _cmd.CommandText = value;
         }
      }

      public int CommandTimeout {
         get {
            CheckDisposed();
            return _cmd.CommandTimeout;
         }
         set {
            CheckDisposed();
            _cmd.CommandTimeout = value;
         }
      }

      public CommandType CommandType {
         get {
            CheckDisposed();
            return _cmd.CommandType;
         }
         set {
            CheckDisposed();
            _cmd.CommandType = value;
         }
      }

      public IDbConnection Connection {
         get {
            CheckDisposed();
            return _cmd.Connection;
         }
         set {
            CheckDisposed();
            _cmd.Connection = value;
         }
      }

      public IDbDataParameter CreateParameter() {
         CheckDisposed();
         return _cmd.CreateParameter();
      }

      public int ExecuteNonQuery() {
         CheckDisposed();
         return _cmd.ExecuteNonQuery();
      }

      public IDataReader ExecuteReader(CommandBehavior behavior) {
         CheckDisposed();
         return _cmd.ExecuteReader(behavior);
      }

      public IDataReader ExecuteReader() {
         CheckDisposed();
         return _cmd.ExecuteReader();
      }

      public object ExecuteScalar() {
         CheckDisposed();
         return _cmd.ExecuteScalar();
      }

      public IDataParameterCollection Parameters {
         get {
            CheckDisposed();
            return _cmd.Parameters;
         }
      }

      public void Prepare() {
         CheckDisposed();
         _cmd.Prepare();
      }

      public IDbTransaction Transaction {
         get {
            CheckDisposed();
            return _cmd.Transaction;
         }
         set {
            CheckDisposed();
            _cmd.Transaction = value;
         }
      }

      public UpdateRowSource UpdatedRowSource {
         get {
            CheckDisposed();
            return _cmd.UpdatedRowSource;
         }
         set {
            CheckDisposed();
            _cmd.UpdatedRowSource = value;
         }
      }

      public void Dispose() {
         _disposed = true;
      }
   }

}
